@@ -148,7 +148,7 @@ pub struct Profile {
     /// all profile attributes
     pub attr: HashMap<String, String>,
     /// map of conversation [ onion => conversation ]
-    pub conversations: HashMap<String, Conversation>,
+    pub conversations: HashMap<i32, Conversation>,
     /// map of servers [ onion => server ]
     pub servers: HashMap<String, Server>,
 }
@@ -289,8 +289,8 @@ impl Profile {
         })
     }
 
-    fn process_conversations(conversations_json: &str) -> Result<HashMap<String, Conversation>, String> {
-        let mut conversations: HashMap<String, Conversation> = HashMap::new();
+    fn process_conversations(conversations_json: &str) -> Result<HashMap<i32, Conversation>, String> {
+        let mut conversations: HashMap<i32, Conversation> = HashMap::new();
         if conversations_json == "null" {
             return Ok(conversations);
         }
@@ -299,7 +299,7 @@ impl Profile {
             Err(e) => return Err(format!("invalid json: {:?}", e)),
         };
         for conversation in conversations_map {
-            conversations.insert(conversation.handle.clone(), conversation);
+            conversations.insert(conversation.identifier, conversation);
         }
         Ok(conversations)
     }
@@ -318,4 +318,12 @@ impl Profile {
         }
         Ok(servers)
     }
+
+    /// Find a conversation_id associated with a remote handle (used for some events with no conversation id like PeerStateChange)
+    pub fn find_conversation_id_by_handle(&self, handle: String) -> Option<i32> {
+        match self.conversations.values().filter(|c| c.handle == handle).next() {
+            Some(conversation) => Some(conversation.identifier),
+            None => None
+        }
+    }
 }
